
# q-expansion of newform 99.2.e.d, downloaded from the LMFDB on 04 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 99
weight = 2
poly_data = [1, 0, 0, -1, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 0, 0, 1, 0, 0], 1], [[0, 1, 0, 0, 0, 1], 1], [[0, 1, 1, 0, -1, 0], 1], [[0, 0, 0, 0, 1, -1], 1], [[0, 1, 0, 0, -1, -1], 1]]

hecke_ring_character_values = [[56, [-1, 1, 0, 0, 0, 0]], [46, [1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 1, -1, -1, 1], [0, 0, 1, 0, 1, 0], [-1, 1, 1, -1, 0, 1], [0, 1, -1, 0, 0, 0], [0, 1, 0, 0, 0, 0], [3, -3, -2, 2, 0, -2], [-1, 0, 0, -2, -1, 0], [-3, 0, 0, -1, 1, 0], [2, -2, 1, -1, 0, -1], [0, 2, -3, -2, -2, 2], [3, -3, 3, -3, 0, 2], [-4, 0, 0, 1, -2, 0], [2, -2, -6, 6, 0, -1], [0, -1, 5, 3, 3, -3], [0, -4, -1, 5, 5, -5], [1, 0, 0, 6, 7, 0], [-7, 7, -3, 3, 0, 3], [0, 7, 1, 0, 0, 0], [-1, 1, -6, 6, 0, -7], [4, 0, 0, 3, 4, 0], [-2, 0, 0, -2, -7, 0], [0, 3, 0, 1, 1, -1], [0, 0, 3, 0, 0, 0], [-4, 0, 0, -8, -2, 0], [0, 1, -4, -4, -4, 4], [0, -7, 2, -7, -7, 7], [0, 0, 8, -8, 0, 5], [-1, 0, 0, 6, 5, 0], [-7, 0, 0, 12, 7, 0], [2, -2, 0, 0, 0, -2], [3, 0, 0, 3, -2, 0], [4, -4, 2, -2, 0, -6], [0, -13, -2, 1, 1, -1], [-1, 1, 8, -8, 0, 7], [3, -3, 11, -11, 0, 9], [0, -5, 9, -15, -15, 15], [2, -2, -7, 7, 0, 0], [1, 0, 0, -9, 0, 0], [-10, 10, 7, -7, 0, 0], [0, 9, -1, 8, 8, -8], [-3, 0, 0, -3, -9, 0], [-10, 0, 0, 2, 8, 0], [0, -7, 3, 0, 0, 0], [-4, 4, 2, -2, 0, 4], [-8, 0, 0, -6, -7, 0], [6, 0, 0, -2, 2, 0], [-1, 1, 0, 0, 0, -11], [0, -5, -8, -1, -1, 1], [0, 14, -10, 9, 9, -9], [12, -12, -1, 1, 0, 1], [-1, 0, 0, 3, 2, 0], [9, -9, 5, -5, 0, 10], [0, 11, -6, 3, 3, -3], [2, 0, 0, 8, -5, 0], [8, -8, 7, -7, 0, -2], [0, -6, 5, -2, -2, 2], [25, 0, 0, 3, -1, 0], [-11, 0, 0, -7, 0, 0], [0, -3, 11, -5, -5, 5], [0, 15, -5, 7, 7, -7], [3, -3, 4, -4, 0, -2], [1, -1, -1, 1, 0, 6], [9, 0, 0, -4, -10, 0], [0, 0, -11, 11, 0, -10], [0, -3, 8, -6, -6, 6], [0, -14, 9, -3, -3, 3], [0, -4, 6, -9, -9, 9], [-8, 8, 3, -3, 0, 0], [15, -15, -7, 7, 0, -1], [0, 16, 0, -4, -4, 4], [0, 11, -1, -9, -9, 9], [-12, 0, 0, -6, -5, 0], [0, 6, -9, -4, -4, 4], [4, -4, -3, 3, 0, 10], [5, 0, 0, 5, -4, 0], [-17, 17, 7, -7, 0, -3], [0, -8, -4, 1, 1, -1], [-11, 0, 0, -3, 12, 0], [-21, 21, -2, 2, 0, -2], [10, -10, -6, 6, 0, 5], [10, -10, -4, 4, 0, -4], [0, 10, 0, 13, 13, -13], [13, 0, 0, -1, 8, 0], [7, 0, 0, -17, -16, 0], [0, 6, -14, 6, 6, -6], [0, 9, 12, 1, 1, -1], [-16, 0, 0, -1, 5, 0], [0, -14, -13, 13, 13, -13], [0, 2, 9, 7, 7, -7], [-1, 1, -7, 7, 0, 7], [14, 0, 0, -2, -2, 0], [0, 6, 8, -20, -20, 20], [0, 0, 0, 13, 21, 0], [18, -18, -13, 13, 0, -1], [-7, 7, 5, -5, 0, 12], [-6, 0, 0, -17, -15, 0], [4, -4, 20, -20, 0, 12], [0, 0, 0, 0, 21, 0], [-10, 0, 0, -1, -8, 0], [-17, 0, 0, -10, -11, 0], [0, -14, 3, 15, 15, -15], [10, 0, 0, -8, -5, 0], [5, -5, -14, 14, 0, -2], [0, -10, -10, 0, 0, 0], [-4, 4, 13, -13, 0, -5], [0, 0, 0, -1, 1, 0], [0, -2, -17, -2, -2, 2], [14, 0, 0, 11, -13, 0], [5, -5, 12, -12, 0, -1], [0, -12, 17, -12, -12, 12], [-2, 2, 13, -13, 0, 5], [4, 0, 0, -5, -14, 0], [1, -1, 2, -2, 0, 12], [0, 0, 4, 11, 11, -11], [4, 0, 0, 10, -1, 0], [0, 10, 3, 8, 8, -8], [-17, 17, -17, 17, 0, -28], [18, 0, 0, -23, -11, 0], [-9, 9, -5, 5, 0, 0], [0, 12, -20, 20, 20, -20], [-7, 7, 11, -11, 0, -2], [0, 8, 7, -21, -21, 21], [0, 13, -15, 20, 20, -20], [-5, 0, 0, 22, 8, 0], [0, -8, 11, 8, 8, -8], [11, 0, 0, -1, -7, 0], [0, 31, -2, -8, -8, 8], [-6, 0, 0, 17, -8, 0], [0, -25, -20, 8, 8, -8], [1, -1, 10, -10, 0, 16], [13, 0, 0, -7, 0, 0], [4, -4, 14, -14, 0, 28], [-5, 5, -19, 19, 0, 5], [16, 0, 0, 19, 1, 0], [20, -20, -22, 22, 0, -13], [4, -4, 8, -8, 0, 28], [-38, 0, 0, 10, 3, 0], [-11, 11, -17, 17, 0, -18], [-1, 1, -12, 12, 0, -24], [12, 0, 0, -10, -5, 0], [7, 0, 0, 14, -8, 0], [0, 4, 1, -5, -5, 5], [-8, 8, -3, 3, 0, -13], [-2, 0, 0, -2, 11, 0], [-23, 0, 0, -9, 15, 0], [0, 0, 23, -24, -24, 24], [0, -9, -28, -3, -3, 3], [0, 20, -3, 0, 0, 0], [-1, 1, 1, -1, 0, 24], [-10, 0, 0, -21, 11, 0], [12, -12, 23, -23, 0, 19], [-3, 0, 0, -13, 0, 0], [18, 0, 0, 11, 27, 0], [-5, 5, 8, -8, 0, -4], [0, -22, -8, 2, 2, -2], [0, -28, 15, -26, -26, 26], [32, 0, 0, -15, -11, 0], [0, -14, 10, 12, 12, -12], [11, 0, 0, 3, 9, 0], [-1, 1, -18, 18, 0, -32], [0, 20, 13, 2, 2, -2], [7, 0, 0, 18, 14, 0], [-10, 10, -1, 1, 0, 1], [4, 0, 0, 11, -11, 0], [-10, 10, -20, 20, 0, -15], [0, 14, 13, -16, -16, 16], [9, 0, 0, 16, 19, 0], [0, 2, 23, -11, -11, 11]]
